/*
** lockstat v1.0
**
** Show file locking state
**
** Copyright (C) 1.10.1997 by Andreas Ley <andreas.ley@kit.edu>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
** This program has been tested on a HP9000/715 with HP-UX A.09.07
** In this environment, neither lint -u nor gcc -Wall -fsyntax-only produce
** any messages. If you encounter any errors or need to make any changes to
** port it to another platform, please contact me.
**
** Version history
**
** Version 1.0 - 1.10.1997
**	Initial version
*/

static char copyright[] = "@(#)Copyright (C) 1997 by Andreas Ley (andreas.ley@kit.edu)";
static char sccsid[] = "@(#)lockstat v1.0 - Show file locking state";


#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>

#ifndef FALSE
#define	FALSE	(0)		/* This is the naked Truth */
#define	TRUE	(1)		/* and this is the Light */
#endif


char	*image;
int	debug=0,quiet=FALSE;


static int lockstat(fname)
char	*fname;
{
	int	fd;

	if (debug)
		(void)fprintf(stderr,"lockstat(\"%s\")\n",fname);
	fd=open(fname,O_WRONLY);
	if (fd<0) {
		(void)fprintf(stderr,"%s: Can't open ",image);
		perror(fname);
		return(1);
	}
	if (lockf(fd,F_TEST,0))
		if (errno==EACCES)
			(void)printf("%s: lockf\n",fname);
		else {
			(void)fprintf(stderr,"%s: Can't lockf ",image);
			perror(fname);
			return(1);
		}
	else
		if (!quiet)
			(void)printf("%s: no lock\n",fname);
	return(close(fd));
}



static void usage()
{
	(void)fprintf(stderr,"Usage: %s [-q] filename [...]\n",image);
	(void)fprintf(stderr,"-q  Quiet, don't show files without locks\n");
	exit(1);
}


int main(argc,argv)
int	argc;
char	*argv[];
{
	int		c;
	extern char	*optarg;
	extern int	optind;

	if ((image=strrchr(argv[0],'/')))
		image++;
	else
		image=argv[0];

	while ((c=getopt(argc,argv,"Dqvh?"))!=EOF)
		switch ((char)c) {
		case 'D':
			debug++;
			break;
		case 'q':
			quiet=TRUE;
			break;
		case 'v':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
			exit(0);
		case 'h':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
		case '?':
			usage();
		}

	if (optind>=argc)
		usage();
	for (;optind<argc;optind++)
		(void)lockstat(argv[optind]);

	return(0);
}
